##############################################################################
#
# Copyright (c) 2002 ZopeChina Corporation (http://www.zopechina.com). All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""Implement the content panels content type."""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent, aq_acquire, aq_inner

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.RevisionManager.RevisionManager import RevisionManager
from Products.RevisionManager import Permissions

factory_type_information = ( {'id': 'RevisionManager',
                            'content_icon': 'revisionmanager.png',
                            'meta_type': 'CMF Revision Manager',
                            'description': """a wrapper of RevisionManager for CMF""", 
                            'product': 'CMFRevisionManager',
                            'factory': 'addCMFRevisionManager',
                            'immediate_view': 'cmfrm_status_form',
                            'actions': ({'id': 'view',
                                        'name': 'View',
                                        'action': 'string:cmfrm_status_form',
                                        'permissions': (CMFCorePermissions.View,)
                                         }
                                      , {'id': 'edit',
                                        'name': 'Edit',
                                        'action': 'string:cmfrm_config_form',
                                        'permissions': (CMFCorePermissions.ModifyPortalContent,)
                                         }
                                      , {'id': 'addformCVS',
                                        'name': 'Add From CVS',
                                        'action': 'string:cmfrm_addfromcvs_form',
                                        'permissions': (CMFCorePermissions.ModifyPortalContent,)
                                         }
                                      , {'id': 'addToCVS',
                                        'name': 'Add To CVS',
                                        'action': 'string:cmfrm_addtocvs_form',
                                        'permissions': (CMFCorePermissions.ModifyPortalContent,),
                                        'visible':0
                                         }
                                     )
                            },
                        )

def addCMFRevisionManager(self
           , id
           , title=''
           , description=''
           ):
    try:
        rm = aq_acquire(self, "RevisionManager")
    except AttributeError:
        rm = None
    if rm is not None:
        raise "Detected a RevisionManager instance."\
              " There already is a RevisionManager instance in this " \
              "folder or one of its parent folders."
    o=CMFRevisionManager(id, title, description)
    self._setObject(id,o)

class CMFRevisionManager(PortalContent,RevisionManager,DefaultDublinCoreImpl):
    """
    CMFRevisionManager
    """

    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    meta_type = 'CMF Revision Manager'
    
    manage_options = RevisionManager.manage_options 
    security = ClassSecurityInfo()

    def __init__(self, id, title='', description=''):
        DefaultDublinCoreImpl.__init__(self)
        RevisionManager.__init__(self)
        self.id=id
        self.title=title
        self.description=description
        RevisionManager.saveSettings(self, {"access_method":"ext", "use_zope_uid":1})

    security.declarePublic("getSupportedMetaTypes")
    def getSupportedFactoryMetaTypes(self):
        """get supported types"""
        return getattr(self, 'supportedTypes', ('File', 'Document', 'Wiki Page'))

    security.declareProtected(Permissions.RMManage, 'setSupportedTypes')
    def setSupportedTypes(self, supportedTypes):
        self.supportedTypes = tuple(supportedTypes)

    security.declareProtected(Permissions.RMAddRemove, 'addFSFileObject')
    def addFSFileObject(self, path, portal_type):
        segments = path.split("/")
        # first make sure that the directory part of the path exists
        # if it doesn't, create as many folders as necessary
        folder = aq_parent(aq_inner(self))
        for id in segments[:-1]:
            subfolder = getattr(aq_base(folder), id, None)
            if subfolder is None:
                folder.manage_addFolder(id)
            folder = getattr(folder, id)

        folder = folder.__of__(self)
        # now add the object
        id = segments[-1]
        folder.invokeFactory(portal_type, id)
        obj = getattr(folder, id)
        
        self._status[path]={} # XXX a hack to avoid RM exception
        self.readFromFS([path])
        self.status([path])
        return 0

InitializeClass(CMFRevisionManager)
